# -*- coding: utf-8 -*-
# Copyright (c) 2012 by Pablo Martín <goinnn@gmail.com>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this software.  If not, see <http://www.gnu.org/licenses/>.

import datetime
import logging

from pygraph.classes.digraph import digraph

from django_detect_cyclic.utils import PyPlete, SCOPE_GLOBAL, format_color

from django_detect_cyclic.graph_utils import find_all_cycle, print_graph, treatment_final_graph
from django_detect_cyclic.utils import get_applications, print_log_info, print_log_error


log = logging.getLogger('django_detect_cyclic.apps_dependence.py')


def create_graph_apps_dependence(file_name, include_apps=None, exclude_apps=None,
                                 exclude_packages=None, verbosity=1, show_modules=False,
                                 remove_isolate_nodes=False, remove_sink_nodes=False,
                                 remove_source_nodes=False, only_cyclic=False, scope=None,
                                 use_colors=True, dotted_scope_local=False,
                                 layout='dot'):
    if only_cyclic:
        remove_isolate_nodes = True
    start_time = datetime.datetime.now()
    gr = create_graph(include_apps, exclude_apps, exclude_packages, verbosity, show_modules, scope, use_colors, dotted_scope_local)
    find_all_cycle(gr, use_colors=use_colors)
    treatment_final_graph(gr, remove_isolate_nodes, remove_sink_nodes, remove_source_nodes,
                              only_cyclic, verbosity=verbosity)
    if file_name:
        print_graph(gr, file_name, layout)
    if print_log_info(verbosity):
        log.info("Duration: %s" % str(datetime.datetime.now() - start_time))
    return gr


def create_graph(include_apps=None, exclude_apps=None, exclude_packages=None, verbosity=0,
                 show_modules=False, scope=None, use_colors=True, dotted_scope_local=False):
    gr = digraph()
    applications = get_applications(include_apps, exclude_apps)
    if not show_modules:
        gr.add_nodes(applications)
    pyplete = PyPlete(scope=scope)
    pyplete_global = None
    if dotted_scope_local:
        pyplete_global = PyPlete(scope=SCOPE_GLOBAL)
    for app_source in applications:
        if print_log_info(verbosity):
            log.info("Analizing %s" % app_source)
        _add_edges_to_package(gr, app_source, app_source, applications, pyplete,
                              exclude_packages, show_modules, verbosity,
                              scope, use_colors, pyplete_global)
    return gr


def _add_edges_to_package(gr, package, app_source, applications,
                          pyplete=None, exclude_packages=None,
                          show_modules=False, verbosity=1,
                          scope=None, use_colors=True,
                          pyplete_global=None):
    pyplete = pyplete or PyPlete(scope=scope)
    package_modules = package.split(".")
    importables_to_app = []
    try:
        pyplete.get_importables_rest_level(importables_to_app, package_modules[0], package_modules[1:], into_module=False)
    except SyntaxError, e:
        if print_log_error(verbosity):
            log.error("\t File: %s SyntaxError %s" % (package_modules, e))

    for importable_to_app, importable_type  in importables_to_app:
        if importable_type == 'package':
            if exclude_packages and importable_to_app in exclude_packages:
                if print_log_info(verbosity):
                    log.info('\t Ignore %s' % importable_to_app)
                continue
            subpackage = '%s.%s' % (package, importable_to_app)
            if subpackage not in applications:
                _add_edges_to_package(gr, subpackage, app_source, applications, pyplete,
                                      exclude_packages=exclude_packages,
                                      show_modules=show_modules,
                                      verbosity=verbosity,
                                      scope=scope,
                                      use_colors=use_colors,
                                      pyplete_global=pyplete_global)
        if importable_type != 'module':
            continue
        if show_modules:
            node = package_modules + [importable_to_app]
            node_source = _add_node_init('.'.join(node), applications)
            if not gr.has_node(node_source):
                _add_node_module(gr, node_source, applications, app_source=app_source, use_colors=use_colors)
        code = pyplete.get_imp_loader_from_path(package_modules[0], package_modules[1:] + [importable_to_app])[0].get_source()
        try:
            imports_code = pyplete.get_pysmell_modules_to_text(code)['POINTERS']
        except SyntaxError, e:
            if print_log_error(verbosity):
                log.error("\t File: %s SyntaxError %s" % (package_modules + [importable_to_app], e))
            continue
        if show_modules:
            for import_code in imports_code.values():
                node_destination = _add_node_init(
                                        _get_module_to_generic_import(gr,
                                                                      import_code.split('.'),
                                                                      pyplete=pyplete,
                                                                      verbosity=verbosity,
                                                                      scope=scope),
                                        applications)
                if node_destination:
                    added = _add_node_module(gr, node_destination, applications,
                                             use_colors=use_colors)
                    if added:
                        style = _edge_style(pyplete_global, gr, node_source,
                                            node_destination, import_code, code)
                        _add_edge(gr, node_source, node_destination, verbosity, style)
        else:
            for import_code in imports_code.values():
                if not import_code.startswith(app_source):
                    for app_destination in applications:
                        if import_code.startswith(app_destination):
                            style = _edge_style(pyplete_global, gr, app_source,
                                                app_destination, import_code, code)
                            _add_edge(gr, app_source, app_destination, verbosity, style)
                            break


def _add_edge(gr, node1, node2, verbosity=1, style="filled"):
    if print_log_info(verbosity):
        log.info('\t %s --> %s' % (node1, node2))
    if not gr.has_edge((node1, node2)):
        gr.add_edge((node1, node2))
        gr.set_edge_label((node1, node2), "(1)")
    else:
        weight = gr.edge_weight((node1, node2)) + 1
        gr.set_edge_weight((node1, node2), weight)
        gr.set_edge_label((node1, node2), "(%s)" % weight)
    attributes = dict(gr.edge_attributes((node1, node2)))
    attributes['style'] = style
    gr.edge_attr[(node1, node2)] = attributes.items()


def _has_scope_global(gr, node1, node2):
    return dict(gr.edge_attributes((node1, node2))).get('style') == 'filled'


def _edge_style(pyplete_global, gr, node1, node2, import_code, code):
    dotted = "filled"
    if pyplete_global and not _has_scope_global(gr, node1, node2):
        imports_code_global = pyplete_global.get_pysmell_modules_to_text(code)['POINTERS']
        if not import_code in imports_code_global.values():
            dotted = "dotted"
    return dotted


#Functions to show modules

def _get_module_to_generic_import(gr, import_code, pyplete=None, verbosity=1, scope=None):
    module = '.'.join(import_code)
    if gr.has_node(module):
        return module
    if len(import_code) == 0:
        return None
    pyplete = pyplete or PyPlete(scope=scope)
    imports = []
    try:
        pyplete.get_importables_rest_level(imports, import_code[0], import_code[1:])
    except SyntaxError, e:
        if print_log_error(verbosity):
            log.error("\t File: %s SyntaxError %s" % (import_code, e))
        return None
    if imports:
        return module
    return _get_module_to_generic_import(gr, import_code[:-1], pyplete=pyplete, verbosity=verbosity, scope=scope)


def _get_app_to_import(node, applications):
    score = len(node)
    candidate = None
    for app in applications:
        if node.startswith(app):
            current_score = len(node) - len(app)
            if current_score < score:
                score = current_score
                candidate = app
    return candidate


def _get_app_colors(app):
    color = 0
    for ch in app:
        color += ord(ch) * int('754321', 16)
    color = int(color % int('ffffff', 16))
    fillcolor = format_color(hex(color))
    fontcolor = format_color(hex(abs(color - int('7fffff', 16))))
    return (fillcolor, fontcolor)


def _add_node_module(gr, node, applications, app_source=None, use_colors=True):
    has_node = gr.has_node(node)
    if not has_node:
        app = app_source or _get_app_to_import(node, applications)
        if app:
            if not has_node:
                gr.add_node(node)
                if use_colors:
                    fillcolor, fontcolor = _get_app_colors(app)
                    gr.add_node_attribute(node, ("fillcolor", fillcolor))
                    gr.add_node_attribute(node, ("color", fontcolor))
                    gr.add_node_attribute(node, ("fontcolor", fontcolor))
                    gr.add_node_attribute(node, ("style", "filled"))
            return True
        return False
    return True


def _add_node_init(node, applications):
    if node in applications:
        return '%s.__init__' % node
    return node

# End functions to show modules



# -*- coding: utf-8 -*-
# Copyright (c) 2012 by Pablo Martín <goinnn@gmail.com>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this software.  If not, see <http://www.gnu.org/licenses/>.


import logging
import sys
try:
    import gv
except ImportError:
    version_info = sys.version_info
    # http://stackoverflow.com/questions/2133767/using-python-graphviz-importerror-no-module-named-gv#comment14947383_2321371
    sys.path.append('/usr/lib/pyshared/python%s.%s' % (version_info[0], version_info[1]))
    import gv

from copy import deepcopy

from pygraph.algorithms.cycles import find_cycle
from pygraph.classes.digraph import digraph
from pygraph.readwrite.dot import write

from django.utils.translation import ugettext_lazy as _

from django_detect_cyclic.utils import print_log_info, format_color

CYCLE_COLOR_SEED = "f8c85c"
CYCLE_LABEL = _('Cycle')
log = logging.getLogger('django_detect_cyclic.graph_utils.py')


def create_graph_test(*args, **kwargs):
    gr = digraph()
    gr.add_nodes(["Portugal", "Spain", "France", "Germany", "Belgium", "Netherlands", "Italy"])
    gr.add_node_attribute("Spain", ("style", "filled"))
    gr.add_node_attribute("Spain", ("fillcolor", "red"))
    gr.add_node_attribute("Spain", ("color", "blue"))
    gr.add_node_attribute("Spain", ("fontcolor", "yellow"))
    gr.add_edge(("Portugal", "Spain"))
    gr.add_edge(("Spain", "France"))
    gr.add_edge_attribute(("Spain", "France"), ("style", "dotted"))
    gr.add_edge(("France", "Portugal"))
    gr.add_edge(("France", "Belgium"))
    gr.add_edge(("France", "Germany"))
    gr.add_edge(("Germany", "France"))
    gr.add_edge(("France", "Italy"))
    gr.add_edge(("Italy", "Belgium"))
    gr.add_edge(("Belgium", "France"))
    gr.add_edge(("Belgium", "Netherlands"))
    gr.add_edge(("Germany", "Belgium"))
    gr.add_edge(("Germany", "Netherlands"))
    return gr


def treatment_final_graph(gr, remove_isolated_nodes=False, remove_sink_nodes=False,
                          remove_source_nodes=False, only_cyclic=False, verbosity=1):
    if only_cyclic:
        for edge, properties in gr.edge_properties.items():
            if not unicode(CYCLE_LABEL) in properties['label']:
                if print_log_info(verbosity):
                    log.info("Remove the edge %s-->%s" % edge)
                gr.del_edge(edge)
    if remove_source_nodes:
        for node, incidence in gr.node_incidence.items():
            if not incidence:
                if print_log_info(verbosity):
                    log.info("Remove the node %s" % node)
                gr.del_node(node)
    if remove_sink_nodes:
        for node, neighbor in gr.node_neighbors.items():
            if not neighbor:
                if print_log_info(verbosity):
                    log.info("Remove the node %s" % node)
                gr.del_node(node)
    if remove_isolated_nodes:
        for node, incidence in gr.node_incidence.items():
            neighbor = gr.node_neighbors.get(node, None)
            if not incidence and not neighbor:
                if print_log_info(verbosity):
                    log.info("Remove the node %s" % node)
                gr.del_node(node)
    return gr


def find_all_cycle(gr, gr_copy=None, use_colors=True, number_cycle=1):
    gr_copy = gr_copy or deepcopy(gr)
    cycle = find_cycle(gr_copy)
    if cycle:
        mark_cycle(gr, cycle, number_cycle, gr_copy, use_colors=use_colors)
        find_all_cycle(gr, gr_copy, use_colors, number_cycle=number_cycle + 1)


def mark_cycle(gr, cycle, number_cycle, gr_copy, use_colors=True):
    i = 0
    cycle_color = None
    while i < len(cycle):
        item = cycle[i]
        try:
            next_item = cycle[i + 1]
        except IndexError:
            next_item = cycle[0]
        weight = gr.edge_weight((item, next_item))
        gr.set_edge_label((item, next_item), "%s %s (%s)" % (unicode(CYCLE_LABEL), number_cycle, weight))
        gr_copy.del_edge((item, next_item))
        i += 1
        if use_colors:
            if not cycle_color:
                cycle_color = format_color(hex((number_cycle * int('369369', 16) + int(CYCLE_COLOR_SEED, 16)) % int('ffffff', 16)))
            gr.add_edge_attribute((item, next_item), ("color", cycle_color))
            gr.add_edge_attribute((item, next_item), ("fontcolor", cycle_color))


def print_graph(gr, file_path, layout='dot'):
    dot = write(gr)
    gvv = gv.readstring(dot)
    gv.layout(gvv, layout)
    format = file_path.split('.')[-1]
    gv.render(gvv, format, file_path)



# -*- coding: utf-8 -*-
# Copyright (c) 2012 by Pablo Martín <goinnn@gmail.com>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this software.  If not, see <http://www.gnu.org/licenses/>.

import os

from django.conf import settings
from django.contrib.auth.decorators import permission_required
from django.http import HttpResponse
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext as _

from django_detect_cyclic.forms import DetectCyclicForm


@permission_required('detect_cyclic.view_graph')
def detect_cyclic(request):
    data = None
    img_src = None
    gr_js = None
    message = None
    if request.method == 'POST':
        data = request.POST
    form = DetectCyclicForm(data=data)
    if form.is_valid():
        gr, file_path = form.detect_cyclic()
        if gr.nodes():
            if file_path:
                img_src = file_path.replace(settings.MEDIA_ROOT, '')
                if img_src and img_src[0] == '/':
                    img_src = img_src[1:]
                img_src = os.path.join(settings.MEDIA_URL, img_src)
        else:
            message = _('Graph empty (without nodes)')
    return render_to_response('detect_cyclic/detect_cyclic.html',
                              {'form': form,
                               'title': _('Detect cyclic'),
                               'img_src': img_src,
                               'gr': gr_js,
                               'message': message},
                              context_instance=RequestContext(request))


@permission_required('detect_cyclic.view_graph')
def save_ajax_svg(request):
    svg_path = request.POST['svgPath'].replace(settings.MEDIA_URL[:-1],
                                               settings.MEDIA_ROOT)
    f = file(svg_path, "w")
    f.write(request.POST['svg'])
    f.close()
    return HttpResponse("ok")



# -*- coding: utf-8 -*-
# Copyright (c) 2012 by Pablo Martín <goinnn@gmail.com>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this software.  If not, see <http://www.gnu.org/licenses/>.

import compiler

from django.conf import settings

from pyplete import PyPlete as PyPleteOriginal
from pysmell.codefinder import CodeFinder, getName, getFuncArgs


SCOPE_GLOBAL = 'global'
DEFAULT_FILENAME = 'detect'
DEFAULT_LAYOUT = 'dot'
DEFAULT_FORMAT = 'svg'


def get_applications(include=None, exclude=None):
    if include:
        return include
    apps = settings.INSTALLED_APPS
    if exclude:
        apps = tuple(set(apps) - set(exclude))
    return apps


class PyPlete(PyPleteOriginal):

    def __init__(self, func_info=None, pythonpath=None, separator='.', scope=None, *args, **kwargs):
        super(PyPlete, self).__init__(func_info=func_info,
                                      pythonpath=pythonpath,
                                      separator=separator,
                                      *args, **kwargs)
        self.scope = scope

    def get_pysmell_modules_to_text(self, text):
        code = compiler.parse(text)

        class GlobalCodeFinder(CodeFinder):

            def visitFunction(self, func):
                self.enterScope(func)
                if self.inClassFunction:
                    if func.name != '__init__':
                        if func.decorators and 'property' in [getName(n) for n in func.decorators]:
                            self.modules.addProperty(self.currentClass, func.name)
                        else:
                            self.modules.addMethod(self.currentClass, func.name,
                                            getFuncArgs(func), func.doc or "")
                    else:
                        self.modules.setConstructor(self.currentClass, getFuncArgs(func))
                elif len(self.scope) == 1:
                    self.modules.addFunction(func.name, getFuncArgs(func,
                                        inClass=False), func.doc or "")

                #self.visit(func.code) Remove this line
                self.exitScope()

        if self.scope == SCOPE_GLOBAL:
            codefinder = GlobalCodeFinder()
        else:
            codefinder = CodeFinder()
        code_walk = compiler.walk(code, codefinder)
        return code_walk.modules


def print_log_info(verbosity):
    return verbosity > 1


def print_log_error(verbosity):
    return verbosity > 0


def compatible_scope(dotted_scope_local, scope_global):
    if dotted_scope_local and scope_global:
        return False
    return True


def format_color(number_hex):
    if number_hex.startswith('0x'):
        number_hex = number_hex[2:]
    if len(number_hex) < 6:
        zeros = '0' * (6 - len(number_hex))
        number_hex = '%s%s' % (zeros, number_hex)
    return '#%s' % number_hex



# -*- coding: utf-8 -*-
# Copyright (c) 2012 by Pablo Martín <goinnn@gmail.com>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this software.  If not, see <http://www.gnu.org/licenses/>.
from optparse import make_option

from django.core.management.base import BaseCommand
from django_detect_cyclic.apps_dependence import create_graph_apps_dependence
from django_detect_cyclic.exceptions import InvalidOptions
from django_detect_cyclic.utils import SCOPE_GLOBAL, DEFAULT_FILENAME, DEFAULT_LAYOUT, DEFAULT_FORMAT, compatible_scope


class Command(BaseCommand):

    option_list = BaseCommand.option_list + (
            make_option('-i', '--include-apps', dest='include_apps',
                    help='Only use these applications to the graph (separated by commas)'),
            make_option('-e', '--exclude-apps', dest='exclude_apps',
                    help='Exclude these apps to the graph (separated by commas)'),
            make_option('-f', '--file-name', dest='file_name', default='%s.%s' % (DEFAULT_FILENAME, DEFAULT_FORMAT),
                    help='The name to the generated path (you can set with path)'),
            make_option('-p', '--exclude-packages', dest='exclude_packages',
                    help='Exclude the next packages. For example migrations,templatetags (separated by commas)'),
            make_option('-c', '--use_colors', dest='use_colors',  action="store_true",
                    help='If you use this option the image will be colored'),
            make_option('-g', '--scope-global', dest='scope_global',  action="store_true",
                    help='The imports into the functions are ignored'),
            make_option('-d', '--dotted-scope-local', dest='dotted_scope_local',  action="store_true",
                    help='The imports into the functions are printing with dotted line'),
            make_option('-s', '--show-modules', dest='show_modules',  action="store_true",
                    help='The nodes now are the modules (by default are the applications)'),
            make_option('-o', '--only-cyclic', dest='only_cyclic',  action="store_true",
                    help='Removes the nodes that do not belong to any cycle'),
            make_option('-r', '--remove-isolate-nodes', dest='remove_isolate_nodes',  action="store_true",
                    help='Removes the isolate nodes'),
            make_option('-k', '--remove-sink-nodes', dest='remove_sink_nodes',  action="store_true",
                    help='Removes the sink nodes'),
            make_option('-a', '--remove-source-nodes', dest='remove_source_nodes',  action="store_true",
                    help='Removes the source nodes'),
            make_option('-l', '--layout', dest='layout', default=DEFAULT_LAYOUT,
                    help='Removes the source nodes'),
    )

    def handle(self, *args, **options):
        include_apps = exclude_apps = exclude_packages = None
        if options['include_apps']:
            include_apps = options['include_apps'].split(',')
        if options['exclude_apps']:
            exclude_apps = options['exclude_apps'].split(',')
        if options['exclude_packages']:
            exclude_packages = options['exclude_packages'].split(',')
        verbosity = int(options['verbosity'])
        remove_isolate_nodes = options['remove_isolate_nodes']
        remove_sink_nodes = options['remove_sink_nodes']
        remove_source_nodes = options['remove_source_nodes']
        only_cyclic = options['only_cyclic']
        show_modules = options['show_modules']
        file_name = options['file_name']
        use_colors = options['use_colors']
        dotted_scope_local = options['dotted_scope_local']
        scope_global = options['scope_global']
        scope = scope_global and SCOPE_GLOBAL or None
        if not compatible_scope(dotted_scope_local, scope_global):
            raise InvalidOptions("The dotted-scope-local and scope-global are incompatibles")
        create_graph_apps_dependence(file_name=file_name,
                                     include_apps=include_apps,
                                     exclude_apps=exclude_apps,
                                     exclude_packages=exclude_packages,
                                     verbosity=verbosity,
                                     show_modules=show_modules,
                                     remove_isolate_nodes=remove_isolate_nodes,
                                     remove_sink_nodes=remove_sink_nodes,
                                     remove_source_nodes=remove_source_nodes,
                                     only_cyclic=only_cyclic,
                                     scope=scope,
                                     use_colors=use_colors,
                                     dotted_scope_local=dotted_scope_local)



# -*- coding: utf-8 -*-
# Copyright (c) 2012 by Pablo Martín <goinnn@gmail.com>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this software.  If not, see <http://www.gnu.org/licenses/>.

import os
from setuptools import setup, find_packages


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

setup(
    name="django-detect-cyclic",
    version="0.0.6",
    author="Pablo Martin",
    author_email="goinnn@gmail.com",
    description="Django application to detect cyclic imports",
    long_description=(read('README.rst') + '\n\n' + read('CHANGES')),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
    ],
    license="LGPL 3",
    keywords="django,imports,cycle,cyclic imports,analyze code",
    url='https://github.com/goinnn/django-detect-cyclic',
    packages=find_packages(where='src'),
    package_dir={'': 'src'},
    include_package_data=True,
    zip_safe=False,
)



# -*- coding: utf-8 -*-
# Copyright (c) 2012 by Pablo Martín <goinnn@gmail.com>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this software.  If not, see <http://www.gnu.org/licenses/>.

import os
import pydot

from django import forms
from django.forms.util import ErrorList
from django.contrib.admin import widgets as widgets_admin
from django.conf import settings
from django.template.defaultfilters import slugify
from django.utils.translation import ugettext_lazy as _

from django_detect_cyclic.apps_dependence import create_graph_apps_dependence
from django_detect_cyclic.utils import DEFAULT_FILENAME, SCOPE_GLOBAL, DEFAULT_FORMAT, DEFAULT_LAYOUT, compatible_scope

FORMAT_SPECIAL = 'svg-js'
LAYOUT_CHOICES = (('circo', 'circo'),
                  ('dot', 'dot'),
                  ('fdp', 'fdp'),
                  ('neato', 'neato'),
                  ('twopi', 'twopi'))


class DetectCyclicForm(forms.Form):

    fieldsets = (
                    (_('Basic options'), {
                        'fields': ('applications', 'show_modules', 'use_colors', 'file_name', 'format', 'layout')
                    }),
                    (_('Scope options'), {
                        'fields': ('scope_global', 'dotted_scope_local'),
                    }),
                    (_('Limits the graph'), {
                        'fields': ('exclude_packages', 'only_cyclic', 'remove_isolate_nodes', 'remove_sink_nodes', 'remove_source_nodes'),
                    }),
                )

    applications = forms.MultipleChoiceField(label=_('Applications'),
                         choices=[(app, app) for app in settings.INSTALLED_APPS],
                         help_text=_('Choose the applications to analize'))
    show_modules = forms.BooleanField(label=_('Show modules'), required=False,
        help_text=_('The nodes now are the modules (by default are the applications)'))
    use_colors = forms.BooleanField(label=_('Use colors'), required=False,
                                    initial=True,
                                    help_text=_('By default the image will be colored. If you uncheck this option the image will be black and white'))
    file_name = forms.CharField(label=_('File name'),
                         initial=DEFAULT_FILENAME,
                         help_text=_('Choose a name to the generated file (without extension)'),
                         required=True)
    format = forms.ChoiceField(label=_('Format'), initial=DEFAULT_FORMAT)
    layout = forms.ChoiceField(label=_('Layout'), choices=LAYOUT_CHOICES, initial=DEFAULT_LAYOUT,
                        help_text=_('Node layout'))
    exclude_packages = forms.CharField(label=_('Exclude packages'), required=False,
                       help_text=_('Exclude the next packages. For example migrations,templatetags (separated by commas)'))
    scope_global = forms.BooleanField(label=_('Scope global'), required=False,
        help_text=_('The imports into the functions are ignored'))
    dotted_scope_local = forms.BooleanField(label=_('Dotted scope local'), initial=True, required=False,
        help_text=_('The imports into the functions are printing with dotted line'))
    only_cyclic = forms.BooleanField(label=_('Only cyclic'), initial=True, required=False,
       help_text=_('Removes the nodes that do not belong to any cycle'))
    remove_isolate_nodes = forms.BooleanField(label=_('Removes isolate nodes'), required=False,
        help_text=_('Removes the isolate nodes (without edgeds)'))
    remove_sink_nodes = forms.BooleanField(label=_('Removes sink nodes'), required=False,
        help_text=_('Removes the sink nodes (without output edges)'))
    remove_source_nodes = forms.BooleanField(label=_('Removes source nodes'), required=False,
        help_text=_('Removes the source nodes (without input edges)'))

    def __init__(self, *args, **kwargs):
        super(DetectCyclicForm, self).__init__(*args, **kwargs)
        if 'south' in settings.INSTALLED_APPS:
            self.fields['exclude_packages'].initial = 'migrations'
        dot = pydot.Dot()
        formats = [(format, format) for format in dot.formats]
        formats.append((FORMAT_SPECIAL, FORMAT_SPECIAL))
        formats.sort()
        self.fields['format'].choices = formats
        applications_label = self.fields['applications'].label
        applications_choices = self.fields['applications'].choices
        applications_initial = [app_value for app_label, app_value in applications_choices
                                              if not app_value.startswith('django.') and app_value != 'django_detect_cyclic']
        self.fields['applications'].initial = applications_initial
        self.fields['applications'].widget = widgets_admin.FilteredSelectMultiple(applications_label,
                                                                                  is_stacked=False,
                                                                                  choices=applications_choices)

    def clean_file_name(self):
        return str(slugify(self.cleaned_data['file_name']))

    def clean_layout(self):
        return str(self.cleaned_data['layout'])

    def clean(self):
        cleaned_data = super(DetectCyclicForm, self).clean()
        dotted_scope_local = cleaned_data['dotted_scope_local']
        scope_global = cleaned_data['scope_global']
        if not compatible_scope(dotted_scope_local, scope_global):
            scope_global_errors = self._errors.get('scope_global', ErrorList())
            scope_global_errors.append(_('This option is incompatible with dotted scope local'))
            self._errors['scope_global'] = scope_global_errors
        return cleaned_data

    def detect_cyclic(self):
        applications = self.cleaned_data['applications']
        exclude_packages = self.cleaned_data['exclude_packages']
        verbosity = 1
        if settings.DEBUG:
            verbosity = 2
        show_modules = self.cleaned_data['show_modules']
        remove_isolate_nodes = self.cleaned_data['remove_isolate_nodes']
        remove_sink_nodes = self.cleaned_data['remove_sink_nodes']
        remove_source_nodes = self.cleaned_data['remove_source_nodes']
        only_cyclic = self.cleaned_data['only_cyclic']
        scope_global = self.cleaned_data['scope_global']
        layout = self.cleaned_data['layout']
        scope = None
        if scope_global:
            scope = SCOPE_GLOBAL
        use_colors = self.cleaned_data['use_colors']
        dotted_scope_local = self.cleaned_data['dotted_scope_local']
        file_name = self.cleaned_data['file_name']
        if file_name:
            graph_dir = os.path.join(settings.MEDIA_ROOT, 'graph')
            if not os.path.isdir(graph_dir):
                os.mkdir(graph_dir)
            format_path = self.cleaned_data['format']
            if format_path == FORMAT_SPECIAL:
                format_path = 'svg'
            file_name = str(os.path.join(graph_dir,
                            '%s.%s' % (file_name, format_path)))
        else:
            use_colors = True
        gr = create_graph_apps_dependence(file_name=file_name, include_apps=applications,
                                        exclude_apps=None, exclude_packages=exclude_packages,
                                        verbosity=verbosity, show_modules=show_modules,
                                        remove_isolate_nodes=remove_isolate_nodes,
                                        remove_sink_nodes=remove_sink_nodes,
                                        remove_source_nodes=remove_source_nodes,
                                        only_cyclic=only_cyclic, scope=scope,
                                        use_colors=use_colors,
                                        dotted_scope_local=dotted_scope_local,
                                        layout=layout)
        return (gr, file_name)

    def __unicode__(self):
        try:
            from formadmin.forms import as_django_admin
            return as_django_admin(self)
        except ImportError:
            return super(DetectCyclicForm, self).__unicode__()
